// ProcTexture.cpp: implementation of the CProcTexture class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wood.h"
#include "ProcTexture.h"
#include "dxutil.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CProcTexture::CProcTexture(LPDIRECT3DDEVICE8 pDev,int width)
{
	// save device pointer
	m_pDev=pDev;

	// save dimensions
	m_width=width;

	// calculate size of image buffer
	m_numTexels=m_width*m_width;

	// create vector based color buffer 
	m_col=new D3DXCOLOR[m_numTexels];
	ZeroMemory(m_col,sizeof(D3DXCOLOR)*m_numTexels);

	// create texture
	HRESULT hr;
	hr=D3DXCreateTexture(m_pDev,m_width,m_width,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,&m_textPtr);
}

CProcTexture::~CProcTexture()
{
	SAFE_RELEASE(m_textPtr);
	SAFE_DELETE(m_col);
}

void CProcTexture::SetTexture(int stage)
{
	m_pDev->SetTexture(stage,m_textPtr);
}

void CProcTexture::Update()
{
	// return if no texture surface pointer

	if (!m_textPtr) 
		return;

	DWORD *colDat=new DWORD[m_numTexels];
	for (int i=0;i<m_numTexels;i++)
		colDat[i]=(DWORD) m_col[i];
	RECT rect;
	rect.left=rect.top=0;
	rect.right=m_width;
	rect.bottom=m_width;
	IDirect3DSurface8 *surfPtr;
	m_textPtr->GetSurfaceLevel(0,&surfPtr);
	D3DXLoadSurfaceFromMemory(surfPtr,NULL,NULL,colDat,D3DFMT_A8R8G8B8,m_width*sizeof(DWORD),
							  NULL,&rect,D3DX_FILTER_NONE,0);
	surfPtr->Release();
	delete colDat;
}

void CProcTexture::Normalize()
{
	int i,j;			// general index counters

	// loop through all the members of the color array and find the extrema

	float maxVal,minVal;
	float *ptr=(float *) m_col;
	minVal=maxVal=*ptr;
	for (i=0;i<m_numTexels;i++) {
		for (j=0;j<4;j++) {
			if (*ptr>maxVal)
				maxVal=*ptr;
			else if (*ptr<minVal)
				minVal=*ptr;
			ptr++;
		}
		ptr++;
	}
	
	// return to caller if nothing to do

	if (minVal>=0.0f&&maxVal<=1.0f)
		return;

	// determine output extrema

	float minOut,maxOut;
	minOut=(minVal<0.0f)?0.0f:minVal;
	maxOut=(maxVal>1.0f)?1.0f:maxVal;

	// determine range of input and output values

	float rangeIn=maxVal-minVal;
	float rangeOut=maxOut-minOut;

	// determine scaling factor

	float scale=rangeOut/rangeIn;

	// loop through teh color values and normalize

	ptr=(float *) m_col;
	for (i=0;i<m_numTexels;i++) {
		for (j=0;j<3;j++) {
			*ptr=(*ptr-minVal)*scale+minOut;
			ptr++;
		}
		ptr++;
	}
}
