// WoodTexture.cpp: implementation of the CWoodTexture class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "wood.h"
#include "WoodTexture.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CWoodTexture::CWoodTexture(LPDIRECT3DDEVICE8 pDev,int width)
//:CProcTexture(pDev,width)
{
	// save device pointer
	m_pDev=pDev;

	// save dimensions
	m_width=width;

	// calculate size of image buffer
	m_numTexels=m_width*m_width;

	// create vector based color buffer 
	m_col=new D3DXCOLOR[m_numTexels];
	ZeroMemory(m_col,sizeof(D3DXCOLOR)*m_numTexels);

	// create texture
	HRESULT hr;
	hr=D3DXCreateTexture(m_pDev,m_width,m_width,1,0,D3DFMT_A8R8G8B8,D3DPOOL_MANAGED,&m_textPtr);
}

void CWoodTexture::CreateTexture(LP_PROCTEXT_WOOD_PARMS parms)
{
	// randomize to the specified seed, and get a few off the top because the 
	// first few calls after srand() are very non-random
	srand(parms->seed);
	rand(); rand(); rand();

	int i,j;

	// take pointer to start of color buffer
	D3DXCOLOR *ptr;
	ptr=m_col;

	// allocate temporary buffer for turbulance map
	float *turb=new float[m_width];

	// calculate slope from specified angle of cut
	float slope=cosf(D3DXToRadian(parms->cutAngle));

	// calculate scaling factor for distance, scale 0->1 to 0->2Pi
	float periodScale=6.2831853f/parms->spacing;

	// generate initial turbulance data
	turb[0]=parms->turbulence-parms->turbulence*2.0f*rand()/(float)RAND_MAX;
	for (i=1;i<m_width;i++) 
		turb[i]=0.5f*(turb[i-1]+parms->turbulence-parms->turbulence*2.0f*rand()/(float)RAND_MAX);

	// loop through the rows
	for (i=0;i<m_width;i++) {

		// calculate Z
		float z=parms->zDist+slope*(i-m_width*0.5f);

		// loop through the columns
		for (j=0;j<m_width;j++) {

			// caluclate Z
			float x=parms->xDist+j-m_width*0.5f;

			// calculate distance from center and offset by turbulence
			float dist=sqrtf(z*z+x*x)+turb[j];

			// calculate position within rings, smoothed
			float val=0.5f*(1.0f+cosf(dist*periodScale));

			// calculate color at this position
			D3DXColorLerp(ptr,&parms->col1,&parms->col2,val);

			// move on to the next pixel in the row
			ptr++;
		}

		// smooth xisting turbulance
		float f=turb[0];
		turb[0]=0.5f*(turb[0]+turb[1]);
		for (j=1;j<m_width-1;j++) {
			float nf=turb[j];
			turb[j]=0.5f*turb[j]+0.25f*f+0.25f*turb[j+1];
			f=nf;
		}
		turb[m_width-1]=0.5f*(turb[m_width-1]+f);

		// add next generation of turbulence
		for (j=0;j<m_width;j++) 
			turb[j]+=parms->turbulence-parms->turbulence*2.0f*rand()/(float)RAND_MAX;
	}

	// de-allocate the turbulence array
	delete turb;

	// update the texture surface using the color array we just filled
	Update();
}

CWoodTexture::~CWoodTexture()
{

}
